#! perl		#### THIS IS A GENERATED FILE. DO NO MODIFY

package ChordPro::Config::Data;

use JSON::XS qw();
use JSON::Relaxed::Parser qw();
use feature qw(state);

our $VERSION = 6.031;

sub config {
    state $pp = JSON::XS->new->utf8
	->boolean_values( $JSON::Boolean::false, $JSON::Boolean::true );

    $pp->decode( <<'EndOfJSON' );
{"a2crd":{"classifier":"pct_chords","infer-titles":true,"tabstop":"8"},"assets":{},"chord-formats":{"common":"%{root|%{}%{qual|%{}}%{ext|%{}}%{bass|/%{}}|%{name}}","nashville":"%{root|%{}%{qual|<sup>%{}</sup>}%{ext|<sup>%{}</sup>}%{bass|/<sub>%{}</sub>}|%{name}}","roman":"%{root|%{}%{qual|<sup>%{}</sup>}%{ext|<sup>%{}</sup>}%{bass|/<sub>%{}</sub>}|%{name}}"},"chordpro":{"chorus":{"recall":{"quote":false,"tag":"","type":""}},"comments":"retain"},"chords":[],"contents":[{"fields":["songindex"],"fold":false,"label":"Table of Contents","line":"%{title}","omit":false,"pageno":"%{page}","template":"stdtoc"},{"fields":["sorttitle","artist"],"fold":false,"label":"Contents by Title","line":"%{title}%{artist| - %{}}","omit":false,"pageno":"%{page}","template":"stdtoc"},{"fields":["artist","sorttitle"],"fold":false,"label":"Contents by Artist","line":"%{artist|%{} - }%{title}","omit":true,"pageno":"%{page}","template":"stdtoc"}],"dates":{"today":{"format":"%A, %B %e, %Y"}},"debug":{"a2crd":"0","abc":"0","assets":"0","chords":"0","config":"0","csv":"0","echo":"0","fonts":"0","images":"0","layout":"0","ly":"0","meta":"0","mma":"0","ops":"0","paths":"0","pp":"0","runtimeinfo":"1","song":"0","songfull":"0","spacing":"0","svg":"0","txtblk":"0","x1":"0","x2":"0","x3":"0"},"delegates":{"abc":{"config":"default","handler":"abc2svg","module":"ABC","preamble":["%%textfont pdf.fonts.text","%%vocalfont pdf.fonts.text","%%gchordfont pdf.fonts.chord"],"preprocess":{"abc":[]},"program":"","type":"image"},"ly":{"config":"default","handler":"ly2svg","module":"Lilypond","preamble":["\\version \"2.21.0\"","\\header { tagline = ##f }"],"type":"image"},"svg":{"handler":"svg2svg","module":"SVG","type":"image"},"textblock":{"handler":"txt2xform","module":"TextBlock","type":"image"}},"diagnostics":{"format":"\"%f\", line %n, %m\n\t%l"},"diagrams":{"show":"all","sorted":false,"suppress":[]},"html":{"styles":{"display":"chordpro.css","print":"chordpro_print.css"}},"include":["guitar"],"instrument":{"description":"","type":""},"latex":{"template_include_path":[],"templates":{"comment":"comment.tt","image":"image.tt","songbook":"songbook.tt"}},"meta":{"_configversion":[6.031]},"metadata":{"autosplit":true,"keys":["title","sorttitle","subtitle","artist","composer","lyricist","arranger","album","copyright","year","key","time","tempo","capo","duration"],"separator":"; ","strict":true},"notes":{"flat":["C",["Db","Des","D♭"],"D",["Eb","Es","Ees","E♭"],"E","F",["Gb","Ges","G♭"],"G",["Ab","As","Aes","A♭"],"A",["Bb","Bes","B♭"],"B"],"movable":false,"sharp":["C",["C#","Cis","C♯"],"D",["D#","Dis","D♯"],"E","F",["F#","Fis","F♯"],"G",["G#","Gis","G♯"],"A",["A#","Ais","A♯"],"B"],"system":"common"},"parser":{"preprocess":{"all":[],"directive":[],"songline":[]}},"pdf":{"back-matter":"","capoheading":"%{capo|Capo: %{}}","chordscolumn":"0","chorus":{"bar":{"color":"foreground","offset":"8","width":"1"},"indent":"0","recall":{"choruslike":false,"quote":false,"tag":"Chorus","type":"comment"},"tag":"Chorus"},"columnspace":"20","corefonts":{"remap":null},"csv":{"fields":[{"meta":"title","name":"title"},{"meta":"pagerange","name":"pages"},{"meta":"sorttitle","name":"sort title"},{"meta":"artist","name":"artists"},{"meta":"composer","name":"composers"},{"meta":"collection","name":"collections"},{"meta":"key_actual","name":"keys"},{"meta":"year","name":"years"},{"name":"my_field","omit":true,"value":"text"}],"separator":";","songsonly":true,"vseparator":"|"},"diagrams":{"align":"left","barwidth":0.8,"dotsize":0.8,"fingers":true,"height":"6","hspace":3.95,"linewidth":0.1,"nutwidth":"5","show":"bottom","vcells":"4","vspace":"3","width":"6"},"even-odd-pages":"1","fontconfig":{"courier":{"":"Courier","bold":"Courier-Bold","bolditalic":"Courier-BoldOblique","italic":"Courier-Oblique"},"dingbats":{"":"ZapfDingbats"},"helvetica":{"":"Helvetica","bold":"Helvetica-Bold","boldoblique":"Helvetica-BoldOblique","oblique":"Helvetica-Oblique"},"mono, monospace":{"":"Courier","bold":"Courier-Bold","bolditalic":"Courier-BoldOblique","italic":"Courier-Oblique"},"sans, sans-serif":{"":"Helvetica","bold":"Helvetica-Bold","bolditalic":"Helvetica-BoldOblique","italic":"Helvetica-Oblique"},"serif":{"":"Times-Roman","bold":"Times-Bold","bolditalic":"Times-BoldItalic","italic":"Times-Italic"},"times":{"":"Times-Roman","bold":"Times-Bold","bolditalic":"Times-BoldItalic","italic":"Times-Italic"}},"fontdir":[],"fonts":{"chord":"Helvetica-Oblique 10","chordfingers":{"file":"ChordProSymbols.ttf","numbercolor":"background"},"comment":{"background":"foreground-light","name":"Helvetica","size":"12"},"comment_box":{"frame":"1","name":"Helvetica","size":"12"},"comment_italic":"Helvetica-Oblique 12","grid":"Helvetica 10","tab":"Courier 10","text":"Times-Roman 12","title":"Times-Bold 14","toc":"Times-Roman 11"},"footspace":"20","formats":{"default":{"background":"","footer":["%{title}","","%{page}"],"subtitle":["","",""],"title":["","",""]},"filler":{"background":"","footer":["","",""],"subtitle":["","",""],"title":["","",""]},"first":{"background":"","footer":["","",""]},"title":{"background":"","footer":["","","%{page}"],"subtitle":["","%{subtitle}",""],"title":["","%{title}",""]}},"front-matter":"","grids":{"cellbar":{"color":"foreground-medium","width":"0"},"show":true,"symbols":{"color":"blue"},"volta":{"color":"blue","span":0.7}},"head-first-only":false,"headspace":"60","info":{"author":"","keywords":"","subject":"","title":"%{title}"},"kbdiagrams":{"align":"left","base":"C","height":"20","hspace":3.95,"keys":"14","linewidth":0.1,"pressed":"foreground-medium","show":"bottom","vspace":0.3,"width":"4"},"labels":{"align":"left","comment":"","width":"auto"},"library":"","marginbottom":"40","marginleft":"40","marginright":"40","margintop":"80","outlines":[{"collapse":false,"fields":["sorttitle","artist"],"fold":false,"label":"By Title","letter":"5","line":"%{title}%{artist| - %{}}"},{"collapse":false,"fields":["artist","sorttitle"],"fold":false,"label":"By Artist","letter":"5","line":"%{artist|%{} - }%{title}"}],"pagealign-songs":"1","papersize":"a4","showlayout":false,"sort-pages":"","spacing":{"chords":1.2,"diagramchords":1.2,"empty":"1","grid":1.2,"lyrics":1.2,"tab":"1","title":1.2,"toc":1.4},"split-marker":["","",""],"theme":{"background":"none","foreground":"black","foreground-light":"grey90","foreground-medium":"grey70"},"titles-directive-ignore":false},"settings":{"chordnames":"strict","chords-canonical":false,"chords-under":false,"choruslabels":true,"columns":"1","decapo":false,"enharmonic-transpose":true,"flowtext":false,"inline-annotations":"%s","inline-chords":false,"lineinfo":true,"lyrics-only":false,"memorize":false,"notenames":false,"strict":true,"suppress-empty-chords":true,"suppress-empty-lyrics":true,"titles":"center","transcode":"","transpose":"0","truesf":false,"wrapindent":"x"},"text":{"chorus":{"recall":{"quote":false,"tag":"","type":""}}},"toc":{"line":"%{title}","order":"page","title":"Table of Contents"},"tuning":["E2","A2","D3","G3","B3","E4"],"user":{"fullname":"","name":""}}
EndOfJSON
}

1;
